@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # The contents of this file are subject to the BitTorrent Open Source License
-# Version 1.0 (the License).  You may not copy or use this file, in either
+# Version 1.1 (the License).  You may not copy or use this file, in either
 # source code or executable form, except in compliance with the License.  You
 # may obtain a copy of the License at http://www.bittorrent.com/license/.
 #
@@ -16,64 +16,61 @@ from __future__ import division
 
 import os
 import sys
-from sha import sha
-from time import time
+import math
+from BTL.hash import sha
+import time
 from threading import Event
+from BitTorrent.translation import _
 
-from BitTorrent.bencode import bencode, bdecode
-from BitTorrent.btformats import check_info
+from BTL.bencode import bencode, bdecode
+from BTL.btformats import check_info
+from BTL.exceptions import str_exc
 from BitTorrent.parseargs import parseargs, printHelp
-from BitTorrent.obsoletepythonsupport import *
+from BTL.obsoletepythonsupport import *
+from BTL.exceptions import str_exc
 from BitTorrent import BTFailure
+from BTL.platform import decode_from_filesystem, get_filesystem_encoding
+from BitTorrent.platform import read_language_file
 
 from khashmir.node import Node
 from khashmir.ktable import KTable
 from khashmir.util import packPeers, compact_peer_info
 
-ignore = ['core', 'CVS', 'Thumbs.db']
+ignore = ['core', 'CVS', 'Thumbs.db', 'desktop.ini']
 
-noncharacter_translate = {}
-for i in range(0xD800, 0xE000):
-    noncharacter_translate[i] = None
-for i in range(0xFDD0, 0xFDF0):
-    noncharacter_translate[i] = None
-for i in (0xFFFE, 0xFFFF):
-    noncharacter_translate[i] = None
+from BTL.ConvertedMetainfo import noncharacter_translate
 
-del i
+def gmtime():
+    return time.mktime(time.gmtime())
 
 def dummy(v):
     pass
 
+minimum_piece_len_pow2 = 15 # minimum piece size is 2**15 = 32KB
+
+def size_to_piece_len_pow2(size):
+    num_pieces_pow2 = 11 # 2**num_pieces_pow2 = desired number of pieces
+    return max(minimum_piece_len_pow2, int(round(math.log(size, 2)) - num_pieces_pow2))
+
 def make_meta_files(url,
                     files,
                     flag=Event(),
                     progressfunc=dummy,
                     filefunc=dummy,
-                    piece_len_pow2=None,
+                    piece_len_pow2=0,
                     target=None,
+                    title=None,
                     comment=None,
-                    filesystem_encoding=None,
+                    safe=None,
+                    content_type=None,  # <---what to do for batch torrents?
                     use_tracker=True,
-                    data_dir = None):
+                    data_dir = None,
+                    url_list = None):
     if len(files) > 1 and target:
         raise BTFailure(_("You can't specify the name of the .torrent file "
                           "when generating multiple torrents at once"))
 
-    if not filesystem_encoding:
-        try:
-            sys.getfilesystemencoding
-        except AttributeError:
-            pass
-        else:
-            filesystem_encoding = sys.getfilesystemencoding()
-        if not filesystem_encoding:
-            filesystem_encoding = 'ascii'
-    try:
-        'a1'.decode(filesystem_encoding)
-    except:
-        raise BTFailure(_('Filesystem encoding "%s" is not supported in this version')
-                        % filesystem_encoding)
+
     files.sort()
     ext = '.torrent'
 
@@ -82,34 +79,47 @@ def make_meta_files(url,
         if not f.endswith(ext):
             togen.append(f)
 
-    total = 0
+    sizes = []
     for f in togen:
-        total += calcsize(f)
+        sizes.append(calcsize(f))
+    total = sum(sizes)
 
     subtotal = [0]
     def callback(x):
         subtotal[0] += x
         progressfunc(subtotal[0] / total)
-    for f in togen:
+    for i, f in enumerate(togen):
         if flag.isSet():
             break
+        if sizes[i] < 1:
+            continue # duh, skip empty files/directories
         t = os.path.split(f)
         if t[1] == '':
             f = t[0]
         filefunc(f)
+        if piece_len_pow2 > 0:
+            my_piece_len_pow2 = piece_len_pow2
+        else:
+            my_piece_len_pow2 = size_to_piece_len_pow2(sizes[i])
+
         if use_tracker:
             make_meta_file(f, url, flag=flag, progress=callback,
-                           piece_len_exp=piece_len_pow2, target=target,
-                           comment=comment, encoding=filesystem_encoding)
+                           piece_len_exp=my_piece_len_pow2, target=target,
+                           title=title, comment=comment, safe=safe,
+                           content_type=content_type,
+                           url_list=url_list)
         else:
             make_meta_file_dht(f, url, flag=flag, progress=callback,
-                           piece_len_exp=piece_len_pow2, target=target,
-                           comment=comment, encoding=filesystem_encoding, data_dir=data_dir)
-            
+                               piece_len_exp=my_piece_len_pow2, target=target,
+                               title=title, comment=comment, safe=safe,
+                               content_type = content_type,
+                               data_dir=data_dir)
+
 
 def make_meta_file(path, url, piece_len_exp, flag=Event(), progress=dummy,
-                   comment=None, target=None, encoding='ascii'):
-    data = {'announce': url.strip(),'creation date': int(time())}
+                   title=None, comment=None, safe=None, content_type=None,
+                   target=None, url_list=None, name=None):
+    data = {'announce': url.strip(), 'creation date': int(gmtime())}
     piece_length = 2 ** piece_len_exp
     a, b = os.path.split(path)
     if not target:
@@ -119,20 +129,30 @@ def make_meta_file(path, url, piece_len_exp, flag=Event(), progress=dummy,
             f = os.path.join(a, b + '.torrent')
     else:
         f = target
-    info = makeinfo(path, piece_length, flag, progress, encoding)
+    info = makeinfo(path, piece_length, flag, progress, name, content_type)
     if flag.isSet():
         return
     check_info(info)
     h = file(f, 'wb')
 
     data['info'] = info
+    lang = read_language_file() or 'en'
+    if lang:
+        data['locale'] = lang
+    if title:
+        data['title'] = title
     if comment:
         data['comment'] = comment
+    if safe:
+        data['safe'] = safe
+    if url_list:
+        data['url-list'] = url_list
     h.write(bencode(data))
     h.close()
-    
-def make_meta_file_dht(path, nodes, piece_len_exp, flag=Event(), progress=dummy,
-                   comment=None, target=None, encoding='ascii', data_dir=None):
+
+def make_meta_file_dht(path, nodes, piece_len_exp, flag=Event(),
+                       progress=dummy, title=None, comment=None, safe=None,
+                       content_type=None, target=None, data_dir=None):
     # if nodes is empty, then get them out of the routing table in data_dir
     # else, expect nodes to be a string of comma seperated <ip>:<port> pairs
     # this has a lot of duplicated code from make_meta_file
@@ -145,7 +165,7 @@ def make_meta_file_dht(path, nodes, piece_len_exp, flag=Event(), progress=dummy,
             f = os.path.join(a, b + '.torrent')
     else:
         f = target
-    info = makeinfo(path, piece_length, flag, progress, encoding)
+    info = makeinfo(path, piece_length, flag, progress, content_type)
     if flag.isSet():
         return
     check_info(info)
@@ -161,12 +181,16 @@ def make_meta_file_dht(path, nodes, piece_len_exp, flag=Event(), progress=dummy,
         nodes = [(node.host, node.port) for node in t.findNodes(info_hash) if node.host != '127.0.0.1']
     else:
         nodes = [(a[0], int(a[1])) for a in [node.strip().split(":") for node in nodes.split(",")]]
-    data = {'nodes': nodes, 'creation date': int(time())}
+    data = {'nodes': nodes, 'creation date': int(gmtime())}
     h = file(f, 'wb')
 
     data['info'] = info
+    if title:
+        data['title'] = title
     if comment:
         data['comment'] = comment
+    if safe:
+        data['safe'] = safe
     h.write(bencode(data))
     h.close()
 
@@ -177,15 +201,22 @@ def calcsize(path):
         total += os.path.getsize(s[1])
     return total
 
-def makeinfo(path, piece_length, flag, progress, encoding):
+def makeinfo(path, piece_length, flag, progress, name = None,
+             content_type = None):  # HEREDAVE. If path is directory,
+                                    # how do we assign content type?
     def to_utf8(name):
-        try:
-            u = name.decode(encoding)
-        except Exception, e:
-            raise BTFailure(_('Could not convert file/directory name "%s" to '
-                              'utf-8 (%s). Either the assumed filesystem '
-                              'encoding "%s" is wrong or the filename contains '
-                              'illegal bytes.') % (name, str(e), encoding))
+        if isinstance(name, unicode):
+            u = name
+        else:
+            try:
+                u = decode_from_filesystem(name)
+            except Exception, e:
+                s = str_exc(e)
+                raise BTFailure(_('Could not convert file/directory name %r to '
+                                  'Unicode (%s). Either the assumed filesystem '
+                                  'encoding "%s" is wrong or the filename contains '
+                                  'illegal bytes.') % (name, s, get_filesystem_encoding()))
+
         if u.translate(noncharacter_translate) != u:
             raise BTFailure(_('File/directory name "%s" contains reserved '
                               'unicode values that do not correspond to '
@@ -207,8 +238,12 @@ def makeinfo(path, piece_length, flag, progress, encoding):
         for p, f in subs:
             pos = 0
             size = os.path.getsize(f)
-            p2 = [to_utf8(name) for name in p]
-            fs.append({'length': size, 'path': p2})
+            p2 = [to_utf8(n) for n in p]
+            if content_type:
+                fs.append({'length': size, 'path': p2,
+                           'content_type' : content_type}) # HEREDAVE. bad for batch!
+            else:
+                fs.append({'length': size, 'path': p2})
             h = file(f, 'rb')
             while pos < size:
                 a = min(size - pos, piece_length - done)
@@ -227,9 +262,16 @@ def makeinfo(path, piece_length, flag, progress, encoding):
             h.close()
         if done > 0:
             pieces.append(sh.digest())
+
+        if name is not None:
+            assert isinstance(name, unicode)
+            name = to_utf8(name)
+        else:
+            name = to_utf8(os.path.split(path)[1])
+
         return {'pieces': ''.join(pieces),
             'piece length': piece_length, 'files': fs,
-            'name': to_utf8(os.path.split(path)[1])}
+            'name': name}
     else:
         size = os.path.getsize(path)
         pieces = []
@@ -245,6 +287,11 @@ def makeinfo(path, piece_length, flag, progress, encoding):
                 p = size
             progress(min(piece_length, size - p))
         h.close()
+        if content_type is not None:
+            return {'pieces': ''.join(pieces),
+                'piece length': piece_length, 'length': size,
+                'name': to_utf8(os.path.split(path)[1]),
+                'content_type' : content_type }
         return {'pieces': ''.join(pieces),
             'piece length': piece_length, 'length': size,
             'name': to_utf8(os.path.split(path)[1])}
