@@ -1,5 +1,5 @@
 # The contents of this file are subject to the BitTorrent Open Source License
-# Version 1.0 (the License).  You may not copy or use this file, in either
+# Version 1.1 (the License).  You may not copy or use this file, in either
 # source code or executable form, except in compliance with the License.  You
 # may obtain a copy of the License at http://www.bittorrent.com/license/.
 #
@@ -8,7 +8,7 @@
 # for the specific language governing rights and limitations under the
 # License.
 
-from BitTorrent.platform import bttime as time
+from BTL.platform import bttime as time
 
 import const
 
@@ -16,8 +16,23 @@ from khash import intify
 from ktable import KTable, K
 from util import unpackNodes
 from krpc import KRPCProtocolError, KRPCSelfNodeError
+from bisect import insort
 
-class ActionBase:
+class NodeWrap(object):
+    def __init__(self, node, target):
+        self.num = target
+        self.node = node
+
+    def __cmp__(self, o):
+        """ this function is for sorting nodes relative to the ID we are looking for """
+        x, y = self.num ^ o.num, self.num ^ self.node.num
+        if x > y:
+            return 1
+        elif x < y:
+            return -1
+        return 0
+        
+class ActionBase(object):
     """ base class for some long running asynchronous proccesses like finding nodes or values """
     def __init__(self, table, target, callback, callLater):
         self.table = table
@@ -25,6 +40,7 @@ class ActionBase:
         self.callLater = callLater
         self.num = intify(target)
         self.found = {}
+        self.foundq = []
         self.queried = {}
         self.queriedip = {}
         self.answered = {}
@@ -32,15 +48,14 @@ class ActionBase:
         self.outstanding = 0
         self.finished = 0
     
-        def sort(a, b, num=self.num):
-            """ this function is for sorting nodes relative to the ID we are looking for """
-            x, y = num ^ a.num, num ^ b.num
-            if x > y:
-                return 1
-            elif x < y:
-                return -1
-            return 0
-        self.sort = sort
+    def sort(self, a, b):
+        """ this function is for sorting nodes relative to the ID we are looking for """
+        x, y = self.num ^ a.num, self.num ^ b.num
+        if x > y:
+            return 1
+        elif x < y:
+            return -1
+        return 0
 
     def shouldQuery(self, node):
         if node.id == self.table.node.id:
@@ -51,6 +66,12 @@ class ActionBase:
             return True
         return False
     
+    def _cleanup(self):
+        self.foundq = None
+        self.found = None
+        self.queried = None
+        self.queriedip = None
+
     def goWithNodes(self, t):
         pass
     
@@ -83,6 +104,7 @@ class FindNode(ActionBase):
             n = self.table.Node().initWithDict(node)
             if not self.found.has_key(n.id):
                 self.found[n.id] = n
+                insort(self.foundq, NodeWrap(n, self.num))
                 self.table.insertNode(n, contacted=0)
         self.schedule()
         
@@ -92,9 +114,8 @@ class FindNode(ActionBase):
         """
         if self.finished:
             return
-        l = self.found.values()
-        l.sort(self.sort)
-        for node in l[:K]:
+        l = [wrapper.node for wrapper in self.foundq[:K]]
+        for node in l:
             if node.id == self.target:
                 self.finished=1
                 return self.callback([node])
@@ -113,14 +134,16 @@ class FindNode(ActionBase):
         if self.outstanding == 0:
             ## all done!!
             self.finished=1
-            self.callLater(self.callback, 0, (l[:K],))
-    
+            self._cleanup()
+            self.callLater(0, self.callback, l[:K])
+
     def makeMsgFailed(self, node):
-        def defaultGotNodes(err, self=self, node=node):
-            self.outstanding = self.outstanding - 1
-            self.schedule()
-        return defaultGotNodes
-    
+        return self._defaultGotNodes
+
+    def _defaultGotNodes(self, err):
+        self.outstanding = self.outstanding - 1
+        self.schedule()
+
     def goWithNodes(self, nodes):
         """
             this starts the process, our argument is a transaction with t.extras being our list of nodes
@@ -131,7 +154,7 @@ class FindNode(ActionBase):
                 continue
             else:
                 self.found[node.id] = node
-        
+                insort(self.foundq, NodeWrap(node, self.num))
         self.schedule()
     
 
@@ -170,6 +193,7 @@ class GetValue(FindNode):
                 if not self.found.has_key(n.id):
                     self.table.insertNode(n)
                     self.found[n.id] = n
+                    insort(self.foundq, NodeWrap(n, self.num))
         elif dict.has_key('values'):
             def x(y, z=self.results):
                 if not z.has_key(y):
@@ -180,16 +204,14 @@ class GetValue(FindNode):
             z = len(dict.get('values', []))
             v = filter(None, map(x, dict.get('values',[])))
             if(len(v)):
-                self.callLater(self.callback, 0, (v,))
+                self.callLater(0, self.callback, v)
         self.schedule()
         
     ## get value
     def schedule(self):
         if self.finished:
             return
-        l = self.found.values()
-        l.sort(self.sort)
-        for node in l[:K]:
+        for node in [wrapper.node for wrapper in self.foundq[:K]]:
             if self.shouldQuery(node):
                 #xxx t.timeout = time.time() + GET_VALUE_TIMEOUT
                 try:
@@ -211,7 +233,8 @@ class GetValue(FindNode):
         if self.outstanding == 0:
             ## all done, didn't find it!!
             self.finished=1
-            self.callLater(self.callback,0, ([],))
+            self._cleanup()
+            self.callLater(0, self.callback, [])
 
     ## get value
     def goWithNodes(self, nodes, found=None):
@@ -224,7 +247,7 @@ class GetValue(FindNode):
                 continue
             else:
                 self.found[node.id] = node
-            
+                insort(self.foundq, NodeWrap(node, self.num))
         self.schedule()
 
 
@@ -270,6 +293,7 @@ class StoreValue(ActionBase):
             except IndexError:
                 if self.outstanding == 0:
                     self.finished = 1
+                    self._cleanup()
                     self.callback(self.stored)
                 return
             else:
@@ -286,8 +310,8 @@ class StoreValue(ActionBase):
                         except KRPCSelfNodeError:
                             pass
                         else:
-                            df.addCallback(self.storedValue,(),{'node':node})
-                            df.addErrback(self.storeFailed, (), {'node':node})
+                            df.addCallback(self.storedValue, node=node)
+                            df.addErrback(self.storeFailed, node=node)
                             self.outstanding += 1
                             num -= 1
                         
@@ -317,9 +341,9 @@ class KeyExpirer:
     def __init__(self, store, callLater):
         self.store = store
         self.callLater = callLater
-        self.callLater(self.doExpire, const.KEINITIAL_DELAY)
+        self.callLater(const.KEINITIAL_DELAY, self.doExpire)
     
     def doExpire(self):
         self.cut = time() - const.KE_AGE
         self.store.expire(self.cut)
-        self.callLater(self.doExpire, const.KE_DELAY)
+        self.callLater(const.KE_DELAY, self.doExpire)
