@@ -1,5 +1,5 @@
 # The contents of this file are subject to the BitTorrent Open Source License
-# Version 1.0 (the License).  You may not copy or use this file, in either
+# Version 1.1 (the License).  You may not copy or use this file, in either
 # source code or executable form, except in compliance with the License.  You
 # may obtain a copy of the License at http://www.bittorrent.com/license/.
 #
@@ -9,13 +9,14 @@
 # License.
 
 import const
+from socket import gethostbyname
 
-from BitTorrent.platform import bttime as time
+from BTL.platform import bttime as time
 
 from sha import sha
-
+import re
 from BitTorrent.defaultargs import common_options, rare_options
-from BitTorrent.RawServer import RawServer
+from BitTorrent.RawServer_twisted import RawServer
 
 from ktable import KTable, K
 from knode import *
@@ -30,20 +31,26 @@ import sys
 import os
 import traceback
 
-from BitTorrent.bencode import bencode, bdecode
+from BTL.bencode import bencode, bdecode
 
 from defer import Deferred
-from random import randrange, sample
+from random import randrange
+from kstore import sample
 
-from threading import Event
+from BTL.stackthreading import Event, Thread
+
+ip_pat = re.compile('[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}')
 
 class KhashmirDBExcept(Exception):
     pass
 
+def foo(bytes):
+    pass
+    
 # this is the base class, has base functionality and find node, no key-value mappings
 class KhashmirBase:
     _Node = KNodeBase
-    def __init__(self, host, port, data_dir, rawserver=None, max_ul_rate=1024, checkpoint=True, errfunc=None):
+    def __init__(self, host, port, data_dir, rawserver=None, max_ul_rate=1024, checkpoint=True, errfunc=None, rlcount=foo, config={'pause':False, 'max_rate_period':20}):
         if rawserver:
             self.rawserver = rawserver
         else:
@@ -52,24 +59,25 @@ class KhashmirBase:
             self.rawserver = RawServer(self.flag, d)
         self.max_ul_rate = max_ul_rate
         self.socket = None
-        self.setup(host, port, data_dir, checkpoint)
-        
-    def setup(self, host, port, data_dir, checkpoint=True):
+        self.config = config
+        self.setup(host, port, data_dir, rlcount, checkpoint)
+
+    def setup(self, host, port, data_dir, rlcount, checkpoint=True):
         self.host = host
         self.port = port
         self.ddir = data_dir
         self.store = KStore()
         self.pingcache = {}
-        self.socket = self.rawserver.create_udpsocket(self.port, self.host, True)
-        self.udp = krpc.hostbroker(self, (self.host, self.port), self.socket, self.rawserver.add_task, self.max_ul_rate)
+        self.socket = self.rawserver.create_udpsocket(self.port, self.host)
+        self.udp = krpc.hostbroker(self, (self.host, self.port), self.socket, self.rawserver.add_task, self.max_ul_rate, self.config, rlcount)
         self._load()
         self.rawserver.start_listening_udp(self.socket, self.udp)
         self.last = time()
         KeyExpirer(self.store, self.rawserver.add_task)
         self.refreshTable(force=1)
         if checkpoint:
-            self.rawserver.add_task(self.findCloseNodes, 30, (lambda a: a, True))
-            self.rawserver.add_task(self.checkpoint, 60, (1,))
+            self.rawserver.add_task(30, self.findCloseNodes, lambda a: a, True)
+            self.rawserver.add_task(60, self.checkpoint, 1)
 
     def Node(self):
         n = self._Node(self.udp.connectionForAddr)
@@ -77,8 +85,9 @@ class KhashmirBase:
         return n
     
     def __del__(self):
-        self.rawserver.stop_listen_udp(self.socket)
-        self.socket.close()
+        if self.socket is not None:
+            self.rawserver.stop_listening_udp(self.socket)
+            self.socket.close()
         
     def _load(self):
         do_load = False
@@ -105,17 +114,16 @@ class KhashmirBase:
             f = open(os.path.join(self.ddir, "routing_table"), 'wb')
             f.write(bencode(d))
             f.close()
-        except:
+        except Exception, e:
             #XXX real error here
             print ">>> unable to dump routing table!", str(e)
             pass
         
         
         if auto:
-            self.rawserver.add_task(self.checkpoint,
-                                    randrange(int(const.CHECKPOINT_INTERVAL * .9),
+            self.rawserver.add_task(randrange(int(const.CHECKPOINT_INTERVAL * .9),
                                               int(const.CHECKPOINT_INTERVAL * 1.1)),
-                                    (1,))
+                                    self.checkpoint, 1)
         
     def _loadRoutingTable(self, nodes):
         """
@@ -137,10 +145,7 @@ class KhashmirBase:
         return l
         
             
-
-    #######
-    #######  LOCAL INTERFACE    - use these methods!
-    def addContact(self, host, port, callback=None):
+    def _addContact(self, host, port, callback=None):
         """
             ping this node and add the contact info to the table on pong!
         """
@@ -151,6 +156,23 @@ class KhashmirBase:
             # our own node
             pass
 
+
+    #######
+    #######  LOCAL INTERFACE    - use these methods!
+    def addContact(self, ip, port, callback=None):
+        """
+            ping this node and add the contact info to the table on pong!
+        """
+        if ip_pat.match(ip):
+            self._addContact(ip, port)
+        else:
+            def go(ip=ip, port=port):
+                ip = gethostbyname(ip)
+                self.rawserver.external_add_task(0, self._addContact, ip, port)
+            t = Thread(target=go)
+            t.start()
+
+
     ## this call is async!
     def findNode(self, id, callback, errback=None):
         """ returns the contact info for node, or the k closest nodes, from the global table """
@@ -170,7 +192,7 @@ class KhashmirBase:
         else:
             # create our search state
             state = FindNode(self, id, d.callback, self.rawserver.add_task)
-            self.rawserver.external_add_task(state.goWithNodes, 0, (nodes,))
+            self.rawserver.external_add_task(0, state.goWithNodes, nodes)
     
     def insertNode(self, n, contacted=1):
         """
@@ -187,8 +209,11 @@ class KhashmirBase:
                 self.checkOldNode(old, n, contacted)
             else:
                 l = self.pingcache.get(old.id, [])
-                l.append((n, contacted))
-                self.pingcache[old.id] = l
+                if len(l) < 10 or contacted:
+                    l.append((n, contacted))
+                    self.pingcache[old.id] = l
+
+                
 
     def checkOldNode(self, old, new, contacted=False):
         ## these are the callbacks used when we ping the oldest node in a bucket
@@ -284,12 +309,15 @@ class KhashmirBase:
             This will allow us to populate our table with nodes on our network closest to our own.
             This is called as soon as we start up with an empty table
         """
-        id = self.node.id[:-1] + chr((ord(self.node.id[-1]) + 1) % 256)
-        self.findNode(id, callback)
+        if not self.config['pause']:
+            id = self.node.id[:-1] + chr((ord(self.node.id[-1]) + 1) % 256)
+            self.findNode(id, callback)
         if auto:
-            self.refreshTable()
-            self.rawserver.external_add_task(self.findCloseNodes, randrange(int(const.FIND_CLOSE_INTERVAL *0.9),
-                                                                   int(const.FIND_CLOSE_INTERVAL *1.1)), (lambda a: True, True))
+            if not self.config['pause']:
+                self.refreshTable()
+            self.rawserver.external_add_task(randrange(int(const.FIND_CLOSE_INTERVAL *0.9),
+                                                       int(const.FIND_CLOSE_INTERVAL *1.1)),
+                                             self.findCloseNodes, lambda a: True, True)
 
     def refreshTable(self, force=0):
         """
@@ -354,13 +382,13 @@ class KhashmirRead(KhashmirBase):
         if searchlocal:
             l = self.retrieveValues(key)
             if len(l) > 0:
-                self.rawserver.external_add_task(callback, 0, (l,))
+                self.rawserver.external_add_task(0, callback, l)
         else:
             l = []
         
         # create our search state
         state = GetValue(self, key, callback, self.rawserver.add_task)
-        self.rawserver.external_add_task(state.goWithNodes, 0, (nodes, l))
+        self.rawserver.external_add_task(0, state.goWithNodes, nodes, l)
 
     def krpc_find_value(self, key, id, _krpc_sender):
         sender = {'id' : id}
@@ -394,7 +422,7 @@ class KhashmirWrite(KhashmirRead):
                     pass
                 response=_storedValueHandler
             action = StoreValue(self, key, value, response, self.rawserver.add_task)
-            self.rawserver.external_add_task(action.goWithNodes, 0, (nodes,))
+            self.rawserver.external_add_task(0, action.goWithNodes, nodes)
             
         # this call is asynch
         self.findNode(key, _storeValueForKey)
